@@ -1,7 +1,8 @@
 import math
+from fontTools.misc.transform import Transform
 from drafting.geometry.geometrical import Geometrical
 from drafting.geometry.point import Point
-from drafting.geometry.primitives import line_intersection, calc_angle
+from drafting.geometry.primitives import line_intersection, calc_angle, polar_coord
 from drafting.interpolation import norm
 
 
@@ -16,6 +17,18 @@ class Line(Geometrical):
         return self.start == l.start and self.end == l.end
 
     __hash__ = object.__hash__
+
+    def point(self, p):
+        if p == "N":
+            return self.pn
+        elif p == "E":
+            return self.pe
+        elif p == "S":
+            return self.ps
+        elif p == "W":
+            return self.pw
+        elif p == "C":
+            return self.mid
     
     @property
     def mid(self):
@@ -82,6 +95,10 @@ class Line(Geometrical):
         b2 = math.pow(self.start.y - self.end.y, 2)
         return math.sqrt(a2 + b2)
     
+    @property
+    def l(self):
+        return self.length()
+    
     def tpx(self, tpx, limit=True):
         x = tpx * math.cos(self.angle())
         y = tpx * math.sin(self.angle())
@@ -101,6 +118,34 @@ class Line(Geometrical):
     def ang(self):
         return self.angle()%math.pi
     
+    def transform(self, t):
+        pts = [self.start, self.end]
+        x1, x2 = [t.transformPoint(pt) for pt in pts]
+        return Line(x1, x2)
+
+    def rotate(self, degrees, point=None):
+        if Transform:
+            t = Transform()
+            if not point:
+                point = self.mid
+            t = t.translate(point.x, point.y)
+            t = t.rotate(math.radians(degrees))
+            t = t.translate(-point.x, -point.y)
+            return self.transform(t)
+        else:
+            raise Exception("fontTools not installed")
+    
+    def bow(self, amt, t=0.5, angle=90):
+        rotated = self.rotate(angle, point=self.t(t))
+        return rotated.tpx(self.length()*0.5+amt)
+    
+    def project(self, pt, dist, angle=90):
+        dx, dy = polar_coord((0, 0), self.ang+math.radians(angle), dist)
+        return self.t(pt).offset(dx, dy)
+    
+    def inset(self, px):
+        return Line(self.tpx(px), self.reverse().tpx(px))
+    
     def extr(self, amt):
         p1, p2 = self
         return Line(p2.i(1+amt, p1), p1.i(1+amt, p2))
@@ -133,6 +178,7 @@ class Line(Geometrical):
         return self.intersection(other)
     
     def join(self, other):
+        from drafting.geometry.rect import Rect
         return Rect.FromPoints(self.start, self.end, other.end, other.start)
     
     def interp(self, x, other):
