@@ -450,6 +450,12 @@ class Rect(Geometrical):
             sw = sh * 1/other.aspect()
         #print(sh, fw, fh, other.aspect())
         return self.take(fh, "mdy")
+    
+    def avg(self):
+        pts = self.cardinals()
+        return Point(
+            sum([p.x for p in pts])/4,
+            sum([p.y for p in pts])/4)
 
     def point(self, eh, ev=Edge.MinX):
         """
@@ -644,19 +650,20 @@ class Rect(Geometrical):
         #if not float(remaining).is_integer():
         #    raise Exception("floating parse")
         auto_count = reified.count("auto")
-        auto_d = remaining / auto_count
-        auto_ds = [auto_d] * auto_count
-        if not auto_d.is_integer():
-            auto_d_floor = math.floor(auto_d)
-            leftover = remaining - auto_d_floor * auto_count
-            for aidx, ad in enumerate(auto_ds):
-                if leftover > 0:
-                    auto_ds[aidx] = auto_d_floor + 1
-                    leftover -= 1
-                else:
-                    auto_ds[aidx] = auto_d_floor
-            #print(auto_ds)
-            #print("NO", auto_d - int(auto_d))
+        if auto_count > 0:
+            auto_d = remaining / auto_count
+            auto_ds = [auto_d] * auto_count
+            if not auto_d.is_integer():
+                auto_d_floor = math.floor(auto_d)
+                leftover = remaining - auto_d_floor * auto_count
+                for aidx, ad in enumerate(auto_ds):
+                    if leftover > 0:
+                        auto_ds[aidx] = auto_d_floor + 1
+                        leftover -= 1
+                    else:
+                        auto_ds[aidx] = auto_d_floor
+                #print(auto_ds)
+                #print("NO", auto_d - int(auto_d))
         res = []
         for r in reified:
             if r == "auto":
@@ -725,4 +732,8 @@ class Rect(Geometrical):
         for w in ws:
             _r, r = r.divide(w, "mxy")
             rs.append(_r)
-        return rs
\ No newline at end of file
+        return rs
+    
+    def to_pen(self):
+        from drafting.pens.draftingpen import DraftingPen
+        return DraftingPen(self)
\ No newline at end of file
